#include <stdio.h>
#include <matrix.h>
#include <vector.h>
#include <math.h>
#include <string.h>
#include "reseniSoustavy.h"

#define M 30
#define METODA 0 //0 - QR rozklad; 1 - DSP library matinvf

static section ("data2")	float Q[M*M];	//vstupni matice -> prejde po roykladu v ortogonalni vystupni
static section ("data1")	float R[M*M];	//horni trojuhelnikova matice po rozkladu
static section ("data2")	float vX[M*M];	//pomocna matice pro vypocty

void reseniSoustavy(float pA[], float pb[], int N, float pX[]) {	
	
	int k,i;	//pomocne promene pro rizeni czklu
	
	float *pQ  = (float *) &Q;	// Pointer na (vstupni matice A) pole Q[][] */ 
    float *pR = (float *) &R;	// Pointer na pole R[][] 
    float *pvX = (float *) &vX;	// Pointer na pole vX[][] 
	
	
	/***************************************************************************/
    /**** Reseni sout. LR pomoci inverzni matice Run-time librery           ****/
    /***************************************************************************/
#if METODA

	matinvf (pA,N,pQ);	//vytvori inverzni matici k A
	transpmf (pQ,N,N,pR);	//transpozice inverzni matice 
	matmmltf (pb,1,N,pR,N,pX);	//vypocet X
	
    /***************************************************************************/
    /**** QR faktorizaci Modified Gram-Schmidt                              ****/
    /***************************************************************************/
#else
    //neni potreba v testech s programem Matlab, protoye uklada cisla po sloupcich
    //transpmf (pA,N,N,pQ);	//transpozice vstupni matice kvuli vhodnejsimu ulozeni dat v pameti (bloky sloupcu po sobe)    
    memcpy(pQ,pA,N*N);
    for (k = 0; k < N; k++) //cyklus o N krocih pro QR faktorizaci
    {
		/* norma vektoru z k-teho sloupce */
    	memcpy(pvX,pQ + k*N,N);	//zkopirovani k-teho vektoru z transformovane matice A
		*(pR + k*N + k) = vecdotf (pvX,pvX,N);	//do X se ulozi soucet prvku vektoru na druhou z k-teho sloupce
    	*(pR + k*N + k) = sqrtf (*(pR + k*N + k));	//do R se na k-ty prvek hlavni diagonly ulozi odmocnina y X
		/*deleni vektoru sklarem */
		for (i = 0; i < N; i++)
		{
			*(pQ + k*N + i) = *(pQ + k*N + i) / *(pR + k*N + k); //postupne se deli cisla k-teho sloupce vektoru Q sklararem z k-te pozice na diagonale a zpet se uklada na stejnou pozici do Q 
		}
 		if (k + 1 < N) //v poslednim cyklu tyto operace nemaji vynam Q, R matice jsou vypocteny
 		{    	
	 		matmmltf (pQ + k*N + N, N - k - 1, N, pQ + k*N, 1, pR + k*N + k + 1); //dopocitani ostatnich koeficentu na k=tem radku v matici R Q[k-ty slpece]'*Q[k+1 sloupce do N-ty slpuce]
	 		
	 	/*	Alternativa k nasledujicim trem instrukcim */
	 		matmmltf (pR + k*N + k + 1, N - k, 1, pQ + k*N, N, pvX);	//vynosobeni Q[k-ty slpuce]*R[k-ty radek,k+1 sloupce do N-ty slpuce]	 			
	 		memcpy(pX,pQ + k*N + N, N*(N - k - 1));	//priprava pro odecteni
	 		matmsubf (pX,pvX,N - k - 1, N, pX);	//odecteni Q[k+1 sloupce do N-ty slpuce]
	 		memcpy (pQ + k*N + N, pX, N*(N - k - 1)); //aktualizace Q */
	 		
	 	/*	Pri nasobeni matmmlt pretykal rozsah promene a prepisuji se i jine casti pameti, jinak je tato metoda stejne rychla jako predchozi
	 		memset(pX + k*N, 0, k*N);
	 		matmmltf (pR + k*N + k + 1, N - k, 1, pQ + k*N, N, pX + k*N + N);	//vynosobeni Q[k-ty slpuce]*R[k-ty radek,k+1 sloupce do N-ty slpuce]	 		
	 		matmsubf (pQ ,pX ,N ,N ,pQ);	//odecteni Q[k+1 sloupce do N-ty slpuce]*/

 		}
	 }
    
    /**** v promenych Q a R jsou cisla rozkladu                             ****/
#endif
	 
    /***************************************************************************/
    /**** Zpetna substituce pro vypocet X                                   ****/
    /***************************************************************************/
    
    matmmltf (pQ,N,N,pb,1,pvX); //vznasobeni Q'*b = vX
    *(pX + N - 1) = *(pvX + N - 1) / *(pR + N*N - 1); //vypocet prvniho prvku
    for (k = N - 1; k > 0; k--) {	//cyklus na vypocet X
    	matmmltf (pX + k, 1, N - k, pR + k*N - N + k, 1, pX + k - 1); //roznasobeni cisel o radek vys resenim Xk (vyjma hlavni diagonaly)
    	*(pX + k - 1) = *(pvX + k - 1) - *(pX + k - 1);	//uprava prave strany predchozim vyledkem
    	*(pX + k - 1) = *(pX + k - 1) / *(pR + k*N - 1 - N + k);	//vypocteni dalsiho Xk			
    }
}
