%SVD (Singular value decomposition)

classdef LU < handle 

    properties (SetAccess = 'private', GetAccess = 'private')
    end % properties
    
    methods
        % Konstruktor   
        function obj = LU() 
        end

            function [lu, pvt] = LUfactor (obj,A)

                [nrow ncol] = size (A);
                if ( nrow ~= ncol )
                   disp ( 'LUfactor error: Square coefficient matrix required' );
                   return;
                end;

                %
                %   initialize row pointers
                %

                for i=1:nrow
                    pvt(i) = i;
                end;
                
                oo = 0;
                for i = 1 : nrow - 1

                %
                %   partial pivoting
                %

                    t =  min ( find ( abs(A(pvt(i:nrow),i)) == max(abs(A(pvt(i:nrow),i))) ) + i-1 );
                    if ( t ~= i )
                       temp = pvt(i);
                       pvt(i) = pvt(t);
                       pvt(t) = temp;
                    end;

                %
                %   terminate if matrix is singular
                %

                    if ( A(pvt(i),i) == 0 ) 
                       disp ( 'LUfactor error: coefficient matrix is singular' );
                       lu = A;
                       return
                    end;

                %
                %   elimination steps
                %

                    for j = i+1 : nrow
                        m = -A(pvt(j),i) / A(pvt(i),i);
                        A(pvt(j),i) = -m;
                        A(pvt(j), i+1:nrow);
                        A(pvt(j), i+1:nrow) = A(pvt(j), i+1:nrow) + m * A(pvt(i), i+1:nrow);
                    end;
                end;
                oo
                lu = A;
            end



            function x = LUsolve (obj, lu, b, pvt )


                [nrow ncol] = size ( lu );

                %
                %   forward substitution
                %

                z(1) = b(pvt(1));
                for i = 2 : nrow
                    z(i) = b(pvt(i)) - sum ( z(1:i-1) .* lu(pvt(i), 1:i-1) );
                end;

                %
                %   back substitution
                %

                x(nrow) = z(nrow) / lu(pvt(nrow), nrow);
                for i = nrow - 1 : -1 : 1
                    x(i) = ( z(i) - sum ( x(i+1:nrow) .* lu(pvt(i), i+1:nrow) ) ) / lu(pvt(i),i);
                end;

            end

    end
end