classdef QR < handle
    
    properties (SetAccess = 'private', GetAccess = 'private')
        V %vektor pro Householder
    end
         
    methods
        % Konstruktor   
        function obj = QR() 
        end
        
        %QR rozklad - Modified Gram-Schmidt Orthogonalization
        function [q,r] = GramSchmidt(obj,A)
            [m, n] = size(A);
            q = zeros(m, n);
            r = zeros(n, n);
            for k = 1:n
                r(k,k) = norm(A(1:m, k));
                if r(k,k) == 0
                    break;
                end               
                q(1:m, k) = A(1:m, k) / r(k,k);
                for j = k+1:n
                    r(k, j) = dot(q(1:m, k), A(1:m, j));
                    A(1:m, j) = A(1:m, j) - r(k, j) * q(1:m, k);
                end           
            end
        end
        
       %upravena verze pro DSP         
        function [Q,R] = GramSchmidtDSP(obj,A);
            [m,n] = size(A);
            Q = A; 
            R=zeros(n);   
            for k = 1:n
                R(k,k) = norm(Q(:,k));
                Q(:,k) = Q(:,k)/R(k,k);
                R(k,k+1:n) = Q(:,k)'*Q(:,k+1:n); %vektor x matice
                Q(:,k+1:n) = Q(:,k+1:n) - Q(:,k)*R(k,k+1:n); %matice - vektor x vektor   
            end
        end
        %QR GS
                
        %QR rozklad - Householder
        function [R] = Householder(obj,A)
            % Compute the QR factorization of real A using Householder reflectors
            % See Trefethen and Bau, Numerical Linear Algebra, Algorithm 10.1 (page 73)
            % The object V is a "cell array": vk (a vector of length m-k+1) is stored in V{k}.
            [m,n] = size(A);
            Q = eye(m);
            for k=1:min(m-1,n)
                ak = A(k:m,k);                                      % vector to be zeroed out
                vk = ak; vk(1) = vk(1) + sign(ak(1))*norm(ak);      % 1. construct vk that defines the reflector
                vk = vk/norm(vk);                                   % 2. normalize vk
                A(k:m,k:n) = A(k:m,k:n) - 2*vk*(vk'*A(k:m,k:n));    % 3. update A
                obj.V{k} = vk;                                      % store vk in a cell array
            end
            R = A;    
        end %QR H
        
        %Vypocet Q z vektoru Vk
        function [Q,y] = Qxb(obj,b)
            NumberOfReflections = length(obj.V);
            y = b;
            mv = length(obj.V{1});
            Q=eye(mv);
            for k = 1:NumberOfReflections,
               F = eye(mv-k+1)-2*(obj.V{k} * obj.V{k}')/norm(obj.V{k})^2;
               Qk = [eye(k-1) zeros(k-1,mv-k+1);zeros(mv-k+1,k-1) F];
               y = Qk * y;
            end       
        end

        %upravena verze pro DSP
        function [R,y] = HouseholderDSP(obj,A,b)
            % Compute the QR factorization of real A using Householder reflectors
            % See Trefethen and Bau, Numerical Linear Algebra, Algorithm 10.1 (page 73)
            % The object V is a "cell array": vk (a vector of length m-k+1) is stored in V{k}.
            [m,n] = size(A);
            y = b;
            Q = eye(m);
            
            for k=1:min(m-1,n)
                ak = A(k:m,k);                                      % vector to be zeroed out
                vk = ak; 
                vk(1) = vk(1) + sign(ak(1))*norm(ak);                  % 1. construct vk that defines the reflector
                vk = vk/norm(vk);                                   % 2. normalize vk
                A(k:m,k:n) = A(k:m,k:n) - 2*vk*(vk'*A(k:m,k:n));    % 3. update A
                F = eye(m-k+1)-2*(vk * vk')/norm(vk)^2;
                Qk = [eye(k-1) zeros(k-1,m-k+1);zeros(m-k+1,k-1) F];
                y = Qk * y;
            end 
            
            R = A;    
        end %QR H
    end 
end %class