%SVD (Singular value decomposition)

classdef SVD < handle 

    properties (SetAccess = 'private', GetAccess = 'private')
    end % properties
    
    methods
        % Konstruktor   
        function obj = SVD() 
        end
        
        %Householderovo zrcadleni na bidiagonalni matici
        function [A, V, U] = upbid(obj,A)
            [m, n] = size(A);
            if m ~= n
                error('Matrix must be square')
            end
            if tril(triu(A),1) == A
                V = eye(n-1);
                U = eye(n-2);
            end
            V = [];
            U = [];
            for k=1:n-1
                x = A(k:n,k);
                v = SVD.Housv(x);
                l = k:n;
                A(l,l) = A(l,l) - 2*v*(v'*A(l,l));
                v = [zeros(k-1,1);v];
                V = [V v];
                if k < n-1
                    x = A(k,k+1:n)';
                    u = SVD.Housv(x);
                    p = 1:n;
                    q = k+1:n;
                    A(p,q) = A(p,q) - 2*(A(p,q)*u)*u';
                    u = [zeros(k,1);u];
                    U = [U u];
                end
            end
        end
        
        
    end  % methods
    
    methods(Static)
        function u = Housv(x)
            % Householder reflection unit vector u from the vector x.
            m = max(abs(x));
            u = x/m;
            if u(1) == 0
                su = 1;
            else
                su = sign(u(1));
            end
            u(1) = u(1)+su*norm(u);
            u = u/norm(u);
            u = u(:);
        end
        
        function Q = Housprod(V)
            % Product Q of several Householder transformations
            % represented by their reflection vectors that are
            % saved in columns of the matrix V.
            [m, n] = size(V);
            Q = eye(m)-2*V(:,n)*V(:,n)';
            for i=n-1:-1:1
                Q = SVD.Houspre(V(:,i),Q);
            end
        end
        
        function P = Houspre(u, A)
            % Product P = H*A, where H is the Householder reflector
            % determined by the vector u and A is a matrix.
            [n, p] = size(A);
            m = length(u);
            if m ~= n
                error('Dimensions of u and A must agree')
            end
            v = u/norm(u);
            v = v(:);
            P = [];
            for j=1:p
                aj = A(:,j);
                P = [P aj-2*v*(v'*aj)];
            end
        end
    end

end %class